// Copyright (c) by respective owners including Yahoo!, Microsoft, and
// individual contributors. All rights reserved. Released under a BSD (revised)
// license as described in the file LICENSE.

#include <boost/test/unit_test.hpp>
#include <boost/test/test_tools.hpp>

#include "cache.h"
#include "vw.h"
#include "test_common.h"

BOOST_AUTO_TEST_CASE(write_and_read_features_from_cache)
{
  auto& vw = *VW::initialize("--quiet");
  VW::example src_ex;
  VW::read_line(vw, &src_ex, "|ns1 example value test |ss2 ex:0.5");

  auto backing_vector = std::make_shared<std::vector<char>>();
  io_buf io_writer;
  io_writer.add_file(VW::io::create_vector_writer(backing_vector));

  VW::details::cache_temp_buffer temp_buffer;
  VW::write_example_to_cache(io_writer, &src_ex, vw.example_parser->lbl_parser, vw.parse_mask, temp_buffer);
  io_writer.flush();

  io_buf io_reader;
  io_reader.add_file(VW::io::create_buffer_view(backing_vector->data(), backing_vector->size()));

  VW::v_array<VW::example*> examples;
  VW::example dest_ex;
  examples.push_back(&dest_ex);
  VW::read_example_from_cache(&vw, io_reader, examples);

  BOOST_CHECK_EQUAL(dest_ex.indices.size(), 2);
  BOOST_CHECK_EQUAL(dest_ex.feature_space['n'].size(), 3);
  BOOST_CHECK_EQUAL(dest_ex.feature_space['s'].size(), 1);

  check_collections_with_float_tolerance(
      src_ex.feature_space['s'].values, dest_ex.feature_space['s'].values, FLOAT_TOL);
  check_collections_exact(src_ex.feature_space['s'].indices, dest_ex.feature_space['s'].indices);
  check_collections_with_float_tolerance(
      src_ex.feature_space['n'].values, dest_ex.feature_space['n'].values, FLOAT_TOL);
  check_collections_exact(src_ex.feature_space['n'].indices, dest_ex.feature_space['n'].indices);

  VW::finish(vw);
}
